@@ -182,7 +182,7 @@ int main (argc, argv)
     FILE *xfile;
     char lscriptpath[512], lscriptpathdel[512];
     char buf[1024];
-    char cmd[512];
+    char cmd[8192];
     char msg[128];
     int baud = 19200;
     int dsubreak = 0;
@@ -257,6 +257,7 @@ int main (argc, argv)
     int agga4 = 0;
     int edac = 0;
     int doft = 0;
+    int fix_sdram_init = 1;
     char *prominit = "prominit_leon3.o";
     int clean_ram0 = 0;
     int clean_ram0_size = 0;
@@ -562,6 +563,10 @@ int main (argc, argv)
 		  {
 		      edac = 1;
 		  }
+                else if (strcmp (argv[stat], "-nofix-sdram") == 0)
+                  {
+                      fix_sdram_init = 0;
+                  }
 		else if (strcmp (argv[stat], "-ft") == 0)
 		  {
 		      doft = 1;
@@ -1106,6 +1111,7 @@ int main (argc, argv)
 	  fprintf (dumpfile,"\n\t.global _uartnr, _sparcleon0, _mpstart, freqkhz\n");
 	  fprintf (dumpfile,"\n\t.global _mpirqsel0, _mpirqsel1, ftsdctrl64_pwr, ftsdctrl64_cfg, ftsdctrl64_edac, ftahbram_cfg\n");
 	  fprintf (dumpfile,"\n\t.global _l1scrub, _l1scrubperiod\n");
+          fprintf (dumpfile,"\n\t.global _fix_sdram_init\n");
       }
     fprintf (dumpfile, "\n\t.global ramsize, _stack, _entry\n");
     fprintf (dumpfile, "\t.global freq, configmsg, bmsg, noinit\n");
@@ -1161,6 +1167,8 @@ int main (argc, argv)
     fprintf (dumpfile, "\t.word\t%d\n", l1scrub);
     fprintf (dumpfile, "_l1scrubperiod:\n");
     fprintf (dumpfile, "\t.word\t%d\n", l1scrubperiod);
+    fprintf (dumpfile, "_fix_sdram_init:\n");
+    fprintf (dumpfile, "\t.word\t%d\n", fix_sdram_init);
     
     
     if (leon)
@@ -1284,6 +1292,19 @@ int main (argc, argv)
 	      }
               }
 	  }
+
+          if (fix_sdram_init) {
+                if (edac_clean_on || edac) { }
+                else if (erc32) {
+                  clean_ram0 = 0x2000000;
+              } else {
+              if (nosram) {
+                  clean_ram0 = 0x40000000;
+              } else {
+                  clean_ram0 = 0x60000000;
+              }
+              }
+          }
 	  
 	  if (nosram)
 	    {
@@ -1949,6 +1970,10 @@ int main (argc, argv)
     strcat (cmd, strlen(prefix) ? prefix : ".");
     strcat (cmd, multidir);
     strcat (cmd,"/prombdinit.o ");
+
+    strcat (cmd, strlen(prefix) ? prefix : ".");
+    strcat (cmd, multidir);
+    strcat (cmd,"/prominit_extra.o ");
     
     if (vverbose)
       strcat (cmd, " -v -Wl,-verbose -Wl,-M ");
@@ -1995,11 +2020,13 @@ int main (argc, argv)
       fflush(stdout);
 #ifdef __MINGW32__
       system("del dump.s");
+      system("del xdump.s");
 #else
       system ("rm -f dump.s");
+      system ("rm -f xdump.s");
 #endif
       if (strlen(lscriptpathdel)) {
-          char c[512];
+          char c[1024];
 #ifdef __MINGW32__
           sprintf(c, "del %s", lscriptpathdel);
 #else
